/**
 * @file
 * Optimized responsive images behavior for Drupal.
 */
(function (Drupal) {
  'use strict';

  // Store processed images for later updates.
  const processedImages = [];

  // Debounce helper to limit the rate of update calls.
  const debounce = (func, delay) => {
    let timeout;
    return function (...args) {
      clearTimeout(timeout);
      timeout = setTimeout(() => func.apply(this, args), delay);
    };
  };

  // Update an image's source and its dimensions based on its parent's container size.
  const updateImage = (image) => {
    // Prevent concurrent updates.
    if (image.dataset.loading) return;
    image.dataset.loading = 'true';

    // Determine multiplier factor.
    let multiplier = parseFloat(image.dataset.multiplier?.replace('x', '')) || 1;
    multiplier = Math.max(multiplier, 1);
    const factor = Math.min(window.devicePixelRatio, multiplier);

    // Get dimensions from the parent container.
    const parent = image.closest('picture');
    if (parent === null) {
      return; // Skip if we can't find a parent picture element
    }
    const containerWidth = parent.offsetWidth;
    const containerHeight = parent.offsetHeight;
    const targetWidth = factor * containerWidth;
    const targetHeight = factor * containerHeight;

    // Always update the image dimensions to match the parent container.
    image.setAttribute('width', containerWidth);
    image.setAttribute('height', containerHeight);

    // Determine the best matching source from data-srcset.
    const sources = image.dataset.srcset.split(',');
    let bestSrc = sources[sources.length - 1].trim().split(' ')[0]; // Fallback to the largest.
    for (let srcInfo of sources) {
      let [src, sizeStr] = srcInfo.trim().split(' ');
      const size = parseInt(sizeStr, 10);
      if (size >= targetWidth) {
        bestSrc = src;
        break;
      }
    }

    // If the current best source is already applied, finish.
    if (image.dataset.currentSrc === bestSrc) {
      image.setAttribute('data-loaded', '1');
      delete image.dataset.loading;
      return;
    }

    // Preload the new best image.
    const tempImg = new Image();
    tempImg.onload = () => {
      // Remove any previously inserted <source> element.
      const existingSource = parent.querySelector('source.easy-lqp-source');
      if (existingSource) {
        parent.removeChild(existingSource);
      }

      // Create and insert a new <source> element.
      const sourceEl = document.createElement("source");
      sourceEl.className = 'easy-lqp-source';
      sourceEl.srcset = bestSrc;
      sourceEl.style.opacity = 0;
      sourceEl.style.transition = "opacity 500ms linear";

      parent.insertBefore(sourceEl, image);
      sourceEl.style.opacity = 1;

      // Update image dimensions again to match the parent's container.
      image.width = containerWidth;
      image.height = containerHeight;

      // Save the applied best source.
      image.dataset.currentSrc = bestSrc;
      image.setAttribute('data-loaded', '1');
      delete image.dataset.loading;
    };
    tempImg.src = bestSrc;
  };

  // Drupal behavior.
  Drupal.behaviors.easyLQP = {
    attach: function (context) {
      // Process each image with a data-srcset attribute only once.
      const images = once('easy-lqp-image', 'img.easy-lqp-image', context);
      if (!images.length) return;

      images.forEach(image => {
        if (!image.dataset.observed) {
          image.dataset.observed = '1';
          processedImages.push(image);
          intersectionObserver.observe(image);
        }
      });
    }
  };

  // IntersectionObserver triggers update when an image enters the viewport.
  const intersectionObserver = new IntersectionObserver(entries => {
    entries.forEach(entry => {
      if (entry.isIntersecting) {
        intersectionObserver.unobserve(entry.target);
        updateImage(entry.target);
        // Observe only the parent container for size changes.
        resizeObserver.observe(entry.target.parentElement);
      }
    });
  }, { rootMargin: '100px' });

  // Single ResizeObserver for the image's parent container.
  const resizeObserver = new ResizeObserver(entries => {
    entries.forEach(entry => {
      // Update all images inside the resized container.
      entry.target.querySelectorAll('img.easy-lqp-image').forEach(image => {
        updateImage(image);
      });
    });
  });

  // Listen for browser window resize events.
  window.addEventListener('resize', (() => {
    processedImages.forEach(image => updateImage(image));
  }));

})(Drupal);
