<?php

namespace Drupal\easy_lqp;

use Drupal\file\FileInterface;

/**
 * Provides an interface for the EasyLqpImagesManager.
 */
interface EasyLqpImagesManagerInterface {

  /**
   * Get all available aspect ratios.
   *
   * @return array
   *   Return options array.
   */
  public function getAspectRatios(): array;

  /**
   * Get all images by a specific aspect ratio as an array.
   *
   * @param string $uri
   *   The uri.
   * @param string $aspect_ratio
   *   The aspect ratio.
   *
   * @return array
   *   Returns style_urls array
   *   consisting of url, width, height & srcset_url.
   */
  public function getImagesByAspectRatio(string $uri, string $aspect_ratio): array;

  /**
   * Get all images, where no aspect ratio should be used.
   *
   * @param string $uri
   *   The uri.
   *
   * @return array
   *   Returns style_urls as an array
   *   consisting of url, width, height, srcset_url.
   */
  public function getImagesByScale(string $uri): array;

  /**
   * Generates a Low  Quality Placeholder version of an image.
   *
   * Creates or updates a database entry with a LQP version of an image.
   *
   * @param \Drupal\file\FileInterface $file
   *   A file.
   * @param string $derivative_url
   *   A url to a Drupal image derivative.
   *
   * @return string|null
   *   Returns the base64 encoded version of an image.
   *   or null if we're updating an image.
   */
  public function generateLqp(FileInterface $file, String $derivative_url): ?string;

  /**
   * Returns the base64 encoded version of an image.
   *
   * If the version does not exist it will attempt to create it.
   *
   * @param \Drupal\file\FileInterface $file
   *   A file.
   * @param string $derivative_url
   *   Url to a Drupal image derivative.
   *
   * @return string
   *   Returns the base64 encoded version of an image.
   */
  public function getLqp(FileInterface $file, String $derivative_url): ?string;

  /**
   * Checks if the file is an image.
   *
   * @param \Drupal\file\FileInterface $file
   *   A file.
   *
   * @return bool
   *   TRUE for an image
   *   FALSE if the file is not permanent or not an image.
   */
  public function validateImage(FileInterface $file): bool;

}
